@@ -4,7 +4,7 @@ import urlparse
 
 from django.core import exceptions, urlresolvers
 from django.test import client, utils
-from django.utils import simplejson as json
+import json
 
 import tastypie
 from tastypie import authorization as tastypie_authorization
@@ -14,6 +14,7 @@ from tastypie_mongoengine import resources as tastypie_mongoengine_resources, te
 from test_project.test_app import documents
 from test_project.test_app.api import resources
 
+
 # TODO: Test set operations
 # TODO: Test bulk operations
 # TODO: Test ordering, filtering
@@ -483,6 +484,23 @@ class BasicTest(test_runner.MongoEngineTestCase):
         response = self.c.get(person1_uri)
         self.assertEqual(response.status_code, 404)
 
+        # Testing x-override
+
+        response = self.c.post(self.resourceListURI('person'), '{"name": "Person X"}', content_type='application/json')
+        self.assertEqual(response.status_code, 201)
+
+        person2_uri = response['location']
+
+        # Testing PATCH with x-override
+
+        response = self.c.post(person2_uri, '{"name": "Person X PATCHED"}', HTTP_X_HTTP_METHOD_OVERRIDE='patch', content_type='application/json')
+        self.assertEqual(response.status_code, 202)
+
+        # Testing DELETE with x-override
+
+        response = self.c.post(person2_uri, HTTP_X_HTTP_METHOD_OVERRIDE='delete')
+        self.assertEqual(response.status_code, 204)
+
     def test_objectclass(self):
         response = self.c.get(self.resourceListURI('personobjectclass'))
 
@@ -624,7 +642,8 @@ class BasicTest(test_runner.MongoEngineTestCase):
         self.assertEqual(response['objects'][1]['resource_uri'], embedded2_uri)
 
         response = self.c.post(embeddedresource_uri, '{"name": "Embedded person 3"}', content_type='application/json')
-        self.assertRedirects(response, embedded3_uri, status_code=201)
+        self.assertEqual(response.status_code, 201)
+        self.assertTrue(response.get('location').endswith(embedded3_uri))
 
         response = self.c.get(embedded3_uri)
         self.assertEqual(response.status_code, 200)
@@ -655,7 +674,9 @@ class BasicTest(test_runner.MongoEngineTestCase):
         self.assertEqual(response.status_code, 404)
 
         response = self.c.post(embeddedresource_uri, '{"name": "Embedded person 4", "optional": "Foobar"}', content_type='application/json')
-        self.assertRedirects(response, embedded4_uri, status_code=201)
+        self.assertEqual(response.status_code, 201)
+        self.assertTrue(response.get('location').endswith(embedded4_uri))
+        # self.assertRedirects(response, embedded4_uri, status_code=201)
 
         response = self.c.get(embedded4_uri)
         self.assertEqual(response.status_code, 200)
@@ -995,7 +1016,9 @@ class BasicTest(test_runner.MongoEngineTestCase):
         response = self.c.put(person1_uri, '{"name": "Person 1a", "strange": "something"}', content_type='application/json; type=strangeperson')
         # Object got replaced, so we get 201 with location, but we do not want a
         # new object, so redirect should match initial resource URL
-        self.assertRedirects(response, person1_uri, status_code=201)
+        self.assertEqual(response.status_code, 201)
+        self.assertTrue(response.get('location').endswith(person1_uri))
+        # self.assertRedirects(response, person1_uri, status_code=201)
 
         response = self.c.get(person1_uri)
         self.assertEqual(response.status_code, 200)
@@ -1017,7 +1040,9 @@ class BasicTest(test_runner.MongoEngineTestCase):
 
         # Changing resource type again (subtype->type)
         response = self.c.put(person1_uri, '{"name": "Person 1c"}', content_type='application/json; type=person')
-        self.assertRedirects(response, person1_uri, status_code=201)
+        self.assertEqual(response.status_code, 201)
+        self.assertTrue(response.get('location').endswith(person1_uri))
+        # self.assertRedirects(response, person1_uri, status_code=201)
 
         response = self.c.get(person1_uri)
         self.assertEqual(response.status_code, 200)
@@ -1028,7 +1053,9 @@ class BasicTest(test_runner.MongoEngineTestCase):
 
         response = self.c.put(person2_uri, '{"name": "Person 2c", "strange": "something"}', content_type='application/json; type=person')
         # Additional fields are ignored
-        self.assertRedirects(response, person2_uri, status_code=201)
+        self.assertEqual(response.status_code, 201)
+        self.assertTrue(response.get('location').endswith(person2_uri))
+        # self.assertRedirects(response, person2_uri, status_code=201)
 
         response = self.c.get(person2_uri)
         self.assertEqual(response.status_code, 200)
@@ -1067,7 +1094,10 @@ class BasicTest(test_runner.MongoEngineTestCase):
         self.assertContains(response, 'field has no data', status_code=400)
 
         response = self.c.post(embeddedresource_uri, '{"name": "Embedded person 3", "strange": "Strange"}', content_type='application/json; type=strangeperson')
-        self.assertRedirects(response, embedded3_uri, status_code=201)
+        self.assertEqual(response.status_code, 201)
+        self.assertTrue(response.get('location').endswith(embedded3_uri))
+        # response.url = response.get('location')
+        # self.assertRedirects(response, embedded3_uri, status_code=201)
 
         response = self.c.get(embedded3_uri)
         self.assertEqual(response.status_code, 200)
@@ -1525,7 +1555,8 @@ class BasicTest(test_runner.MongoEngineTestCase):
         self.assertContains(response, 'Field is required', status_code=400)
 
     def test_readonly_embedded(self):
-        response = self.c.post(self.resourceListURI('readonlyparent'), '{"name": "A readonly embedded test"}', content_type='application/json')
+        uri = self.resourceListURI('readonlyparent')
+        response = self.c.post(uri, '{"name": "A readonly embedded test"}', content_type='application/json')
         self.assertEqual(response.status_code, 201)
 
         document_uri = response['location']
